# Copyright (c) 2016 VMware, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
import mock
from oslo_utils import units
from oslo_vmware.objects import datastore
from oslo_vmware import vim_util

from os_brick import exception
from os_brick.initiator.connectors import vmware
from os_brick.tests.initiator import test_connector


@ddt.ddt
class VmdkConnectorTestCase(test_connector.ConnectorTestCase):

    IP = '127.0.0.1'
    PORT = 443
    USERNAME = 'username'
    PASSWORD = 'password'
    API_RETRY_COUNT = 3
    TASK_POLL_INTERVAL = 5.0
    CA_FILE = "/etc/ssl/rui-ca-cert.pem"
    TMP_DIR = "/vmware-tmp"
    IMG_TX_TIMEOUT = 10

    VMDK_CONNECTOR = vmware.VmdkConnector

    def setUp(self):
        super(VmdkConnectorTestCase, self).setUp()

        self._connector = vmware.VmdkConnector(None)
        self._connector._ip = self.IP
        self._connector._port = self.PORT
        self._connector._username = self.USERNAME
        self._connector._password = self.PASSWORD
        self._connector._api_retry_count = self.API_RETRY_COUNT
        self._connector._task_poll_interval = self.TASK_POLL_INTERVAL
        self._connector._ca_file = self.CA_FILE
        self._connector._insecure = True
        self._connector._tmp_dir = self.TMP_DIR
        self._connector._timeout = self.IMG_TX_TIMEOUT

    def test_load_config(self):
        config = {
            'vmware_host_ip': 'localhost',
            'vmware_host_port': 1234,
            'vmware_host_username': 'root',
            'vmware_host_password': 'pswd',
            'vmware_api_retry_count': 1,
            'vmware_task_poll_interval': 1.0,
            'vmware_ca_file': None,
            'vmware_insecure': False,
            'vmware_tmp_dir': '/tmp',
            'vmware_image_transfer_timeout_secs': 5,
        }
        self._connector._load_config({'config': config})

        self.assertEqual('localhost', self._connector._ip)
        self.assertEqual(1234, self._connector._port)
        self.assertEqual('root', self._connector._username)
        self.assertEqual('pswd', self._connector._password)
        self.assertEqual(1, self._connector._api_retry_count)
        self.assertEqual(1.0, self._connector._task_poll_interval)
        self.assertIsNone(self._connector._ca_file)
        self.assertFalse(self._connector._insecure)
        self.assertEqual('/tmp', self._connector._tmp_dir)
        self.assertEqual(5, self._connector._timeout)

    @mock.patch('oslo_vmware.api.VMwareAPISession')
    def test_create_session(self, session):
        session.return_value = mock.sentinel.session

        ret = self._connector._create_session()

        self.assertEqual(mock.sentinel.session, ret)
        session.assert_called_once_with(
            self._connector._ip,
            self._connector._username,
            self._connector._password,
            self._connector._api_retry_count,
            self._connector._task_poll_interval,
            port=self._connector._port,
            cacert=self._connector._ca_file,
            insecure=self._connector._insecure)

    @mock.patch('oslo_utils.fileutils.ensure_tree')
    @mock.patch('tempfile.mkstemp')
    @mock.patch('os.close')
    def test_create_temp_file(
            self, close, mkstemp, ensure_tree):
        fd = mock.sentinel.fd
        tmp = mock.sentinel.tmp
        mkstemp.return_value = (fd, tmp)

        prefix = ".vmdk"
        suffix = "test"
        ret = self._connector._create_temp_file(prefix=prefix, suffix=suffix)

        self.assertEqual(tmp, ret)
        ensure_tree.assert_called_once_with(self._connector._tmp_dir)
        mkstemp.assert_called_once_with(dir=self._connector._tmp_dir,
                                        prefix=prefix,
                                        suffix=suffix)
        close.assert_called_once_with(fd)

    @mock.patch('os_brick.initiator.connectors.vmware.open', create=True)
    @mock.patch('oslo_vmware.image_transfer.copy_stream_optimized_disk')
    def test_download_vmdk(self, copy_disk, file_open):
        file_open_ret = mock.Mock()
        tmp_file = mock.sentinel.tmp_file
        file_open_ret.__enter__ = mock.Mock(return_value=tmp_file)
        file_open_ret.__exit__ = mock.Mock(return_value=None)
        file_open.return_value = file_open_ret

        tmp_file_path = mock.sentinel.tmp_file_path
        session = mock.sentinel.session
        backing = mock.sentinel.backing
        vmdk_path = mock.sentinel.vmdk_path
        vmdk_size = mock.sentinel.vmdk_size
        self._connector._download_vmdk(
            tmp_file_path, session, backing, vmdk_path, vmdk_size)

        file_open.assert_called_once_with(tmp_file_path, 'wb')
        copy_disk.assert_called_once_with(None,
                                          self._connector._timeout,
                                          tmp_file,
                                          session=session,
                                          host=self._connector._ip,
                                          port=self._connector._port,
                                          vm=backing,
                                          vmdk_file_path=vmdk_path,
                                          vmdk_size=vmdk_size)

    def _create_connection_properties(self):
        return {'volume_id': 'ed083474-d325-4a99-b301-269111654f0d',
                'volume': 'ref-1',
                'vmdk_path': '[ds] foo/bar.vmdk',
                'vmdk_size': units.Gi,
                'datastore': 'ds-1',
                'datacenter': 'dc-1',
                }

    @mock.patch.object(VMDK_CONNECTOR, '_load_config')
    @mock.patch.object(VMDK_CONNECTOR, '_create_session')
    @mock.patch.object(VMDK_CONNECTOR, '_create_temp_file')
    @mock.patch('oslo_vmware.vim_util.get_moref')
    @mock.patch.object(VMDK_CONNECTOR, '_download_vmdk')
    @mock.patch('os.path.getmtime')
    def test_connect_volume(
            self, getmtime, download_vmdk, get_moref, create_temp_file,
            create_session, load_config):
        session = mock.Mock()
        create_session.return_value = session

        tmp_file_path = mock.sentinel.tmp_file_path
        create_temp_file.return_value = tmp_file_path

        backing = mock.sentinel.backing
        get_moref.return_value = backing

        last_modified = mock.sentinel.last_modified
        getmtime.return_value = last_modified

        props = self._create_connection_properties()
        ret = self._connector.connect_volume(props)

        self.assertEqual(tmp_file_path, ret['path'])
        self.assertEqual(last_modified, ret['last_modified'])
        load_config.assert_called_once_with(props)
        create_session.assert_called_once_with()
        create_temp_file.assert_called_once_with(
            suffix=".vmdk", prefix=props['volume_id'])
        download_vmdk.assert_called_once_with(
            tmp_file_path, session, backing, props['vmdk_path'],
            props['vmdk_size'])
        session.logout.assert_called_once_with()

    @ddt.data((None, False), ([mock.sentinel.snap], True))
    @ddt.unpack
    def test_snapshot_exists(self, snap_list, exp_return_value):
        snapshot = mock.Mock(rootSnapshotList=snap_list)
        session = mock.Mock()
        session.invoke_api.return_value = snapshot

        backing = mock.sentinel.backing
        ret = self._connector._snapshot_exists(session, backing)

        self.assertEqual(exp_return_value, ret)
        session.invoke_api.assert_called_once_with(
            vim_util, 'get_object_property', session.vim, backing, 'snapshot')

    def test_create_temp_ds_folder(self):
        session = mock.Mock()
        ds_folder_path = mock.sentinel.ds_folder_path
        dc_ref = mock.sentinel.dc_ref
        self._connector._create_temp_ds_folder(session, ds_folder_path, dc_ref)

        session.invoke_api.assert_called_once_with(
            session.vim,
            'MakeDirectory',
            session.vim.service_content.fileManager,
            name=ds_folder_path,
            datacenter=dc_ref)

    @mock.patch('oslo_vmware.objects.datastore.get_datastore_by_ref')
    @mock.patch.object(VMDK_CONNECTOR, '_create_temp_ds_folder')
    @mock.patch('os_brick.initiator.connectors.vmware.open', create=True)
    @mock.patch.object(VMDK_CONNECTOR, '_upload_vmdk')
    @mock.patch('os.path.getsize')
    def test_disconnect(
            self, getsize, upload_vmdk, file_open, create_temp_ds_folder,
            get_ds_by_ref):
        ds_ref = mock.sentinel.ds_ref
        ds_name = 'datastore-1'
        dstore = datastore.Datastore(ds_ref, ds_name)
        get_ds_by_ref.return_value = dstore

        file_open_ret = mock.Mock()
        tmp_file = mock.sentinel.tmp_file
        file_open_ret.__enter__ = mock.Mock(return_value=tmp_file)
        file_open_ret.__exit__ = mock.Mock(return_value=None)
        file_open.return_value = file_open_ret

        dc_name = mock.sentinel.dc_name
        delete_task = mock.sentinel.delete_vdisk_task
        copy_task = mock.sentinel.copy_vdisk_task
        delete_file_task = mock.sentinel.delete_file_task
        session = mock.Mock()
        session.invoke_api.side_effect = [
            dc_name, delete_task, copy_task, delete_file_task]

        getsize.return_value = units.Gi

        tmp_file_path = '/tmp/foo.vmdk'
        dc_ref = mock.sentinel.dc_ref
        vmdk_path = mock.sentinel.vmdk_path
        self._connector._disconnect(
            tmp_file_path, session, ds_ref, dc_ref, vmdk_path)

        tmp_folder_path = self._connector.TMP_IMAGES_DATASTORE_FOLDER_PATH
        ds_folder_path = '[%s] %s' % (ds_name, tmp_folder_path)
        create_temp_ds_folder.assert_called_once_with(
            session, ds_folder_path, dc_ref)
        file_open.assert_called_once_with(tmp_file_path, "rb")

        self.assertEqual(
            mock.call(vim_util, 'get_object_property', session.vim, dc_ref,
                      'name'), session.invoke_api.call_args_list[0])

        exp_rel_path = '%s/foo.vmdk' % tmp_folder_path
        upload_vmdk.assert_called_once_with(
            tmp_file, self._connector._ip, self._connector._port, dc_name,
            ds_name, session.vim.client.options.transport.cookiejar,
            exp_rel_path, units.Gi, self._connector._ca_file,
            self._connector._timeout)

        disk_mgr = session.vim.service_content.virtualDiskManager
        self.assertEqual(
            mock.call(session.vim, 'DeleteVirtualDisk_Task', disk_mgr,
                      name=vmdk_path, datacenter=dc_ref),
            session.invoke_api.call_args_list[1])
        self.assertEqual(mock.call(delete_task),
                         session.wait_for_task.call_args_list[0])

        src = '[%s] %s' % (ds_name, exp_rel_path)
        self.assertEqual(
            mock.call(session.vim, 'CopyVirtualDisk_Task', disk_mgr,
                      sourceName=src, sourceDatacenter=dc_ref,
                      destName=vmdk_path, destDatacenter=dc_ref),
            session.invoke_api.call_args_list[2])
        self.assertEqual(mock.call(copy_task),
                         session.wait_for_task.call_args_list[1])

        file_mgr = session.vim.service_content.fileManager
        self.assertEqual(
            mock.call(session.vim, 'DeleteDatastoreFile_Task', file_mgr,
                      name=src, datacenter=dc_ref),
            session.invoke_api.call_args_list[3])
        self.assertEqual(mock.call(delete_file_task),
                         session.wait_for_task.call_args_list[2])

    @mock.patch('os.path.exists')
    def test_disconnect_volume_with_missing_temp_file(self, path_exists):
        path_exists.return_value = False

        path = mock.sentinel.path
        self.assertRaises(exception.NotFound,
                          self._connector.disconnect_volume,
                          mock.ANY,
                          {'path': path})
        path_exists.assert_called_once_with(path)

    @mock.patch('os.path.exists')
    @mock.patch('os.path.getmtime')
    @mock.patch.object(VMDK_CONNECTOR, '_disconnect')
    @mock.patch('os.remove')
    def test_disconnect_volume_with_unmodified_file(
            self, remove, disconnect, getmtime, path_exists):
        path_exists.return_value = True

        mtime = 1467802060
        getmtime.return_value = mtime

        path = mock.sentinel.path
        self._connector.disconnect_volume(mock.ANY, {'path': path,
                                                     'last_modified': mtime})

        path_exists.assert_called_once_with(path)
        getmtime.assert_called_once_with(path)
        disconnect.assert_not_called()
        remove.assert_called_once_with(path)

    @mock.patch('os.path.exists')
    @mock.patch('os.path.getmtime')
    @mock.patch.object(VMDK_CONNECTOR, '_load_config')
    @mock.patch.object(VMDK_CONNECTOR, '_create_session')
    @mock.patch('oslo_vmware.vim_util.get_moref')
    @mock.patch.object(VMDK_CONNECTOR, '_snapshot_exists')
    @mock.patch.object(VMDK_CONNECTOR, '_disconnect')
    @mock.patch('os.remove')
    def test_disconnect_volume(
            self, remove, disconnect, snapshot_exists, get_moref,
            create_session, load_config, getmtime, path_exists):
        path_exists.return_value = True

        mtime = 1467802060
        getmtime.return_value = mtime

        session = mock.Mock()
        create_session.return_value = session

        snapshot_exists.return_value = False

        backing = mock.sentinel.backing
        ds_ref = mock.sentinel.ds_ref
        dc_ref = mock.sentinel.dc_ref
        get_moref.side_effect = [backing, ds_ref, dc_ref]

        props = self._create_connection_properties()
        path = mock.sentinel.path
        self._connector.disconnect_volume(props, {'path': path,
                                                  'last_modified': mtime - 1})

        path_exists.assert_called_once_with(path)
        getmtime.assert_called_once_with(path)
        load_config.assert_called_once_with(props)
        create_session.assert_called_once_with()
        snapshot_exists.assert_called_once_with(session, backing)
        disconnect.assert_called_once_with(
            path, session, ds_ref, dc_ref, props['vmdk_path'])
        remove.assert_called_once_with(path)
        session.logout.assert_called_once_with()



# Copyright 2016 Cloudbase Solutions Srl
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
import mock
import six

from os_brick import exception
from os_brick.initiator.windows import base as base_win_conn
from os_brick.tests.windows import fake_win_conn
from os_brick.tests.windows import test_base


@ddt.ddt
class BaseWindowsConnectorTestCase(test_base.WindowsConnectorTestBase):
    def setUp(self):
        super(BaseWindowsConnectorTestCase, self).setUp()

        self._diskutils = mock.Mock()

        self._connector = fake_win_conn.FakeWindowsConnector()
        self._connector._diskutils = self._diskutils

    @ddt.data({},
              {'feature_available': True},
              {'feature_available': False, 'enforce_multipath': True})
    @ddt.unpack
    @mock.patch.object(base_win_conn.utilsfactory, 'get_hostutils')
    def test_check_multipath_support(self, mock_get_hostutils,
                                     feature_available=True,
                                     enforce_multipath=False):
        mock_hostutils = mock_get_hostutils.return_value
        mock_hostutils.check_server_feature.return_value = feature_available
        check_mpio = base_win_conn.BaseWindowsConnector.check_multipath_support

        if feature_available or not enforce_multipath:
            multipath_support = check_mpio(
                enforce_multipath=enforce_multipath)
            self.assertEqual(feature_available, multipath_support)
        else:
            self.assertRaises(exception.BrickException,
                              check_mpio,
                              enforce_multipath=enforce_multipath)
        mock_hostutils.check_server_feature.assert_called_once_with(
            mock_hostutils.FEATURE_MPIO)

    @ddt.data({}, {'mpio_requested': False}, {'mpio_available': True})
    @mock.patch.object(base_win_conn.BaseWindowsConnector,
                       'check_multipath_support')
    @ddt.unpack
    def test_get_connector_properties(self, mock_check_mpio,
                                      mpio_requested=True,
                                      mpio_available=True):
        mock_check_mpio.return_value = mpio_available
        enforce_multipath = False

        props = base_win_conn.BaseWindowsConnector.get_connector_properties(
            multipath=mpio_requested,
            enforce_multipath=enforce_multipath)
        self.assertEqual(mpio_requested and mpio_available,
                         props['multipath'])
        if mpio_requested:
            mock_check_mpio.assert_called_once_with(enforce_multipath)

    def test_get_scsi_wwn(self):
        mock_get_uid_and_type = self._diskutils.get_disk_uid_and_uid_type
        mock_get_uid_and_type.return_value = (mock.sentinel.disk_uid,
                                              mock.sentinel.uid_type)

        scsi_wwn = self._connector._get_scsi_wwn(mock.sentinel.dev_num)
        expected_wwn = '%s%s' % (mock.sentinel.uid_type,
                                 mock.sentinel.disk_uid)
        self.assertEqual(expected_wwn, scsi_wwn)
        mock_get_uid_and_type.assert_called_once_with(mock.sentinel.dev_num)

    @ddt.data(None, IOError)
    @mock.patch.object(six.moves.builtins, 'open')
    def test_check_valid_device(self, exc, mock_open):
        mock_open.side_effect = exc

        valid_device = self._connector.check_valid_device(
            mock.sentinel.dev_path)
        self.assertEqual(not exc, valid_device)

        mock_open.assert_any_call(mock.sentinel.dev_path, 'r')
        mock_read = mock_open.return_value.__enter__.return_value.read
        if not exc:
            mock_read.assert_called_once_with(1)

    def test_check_device_paths(self):
        # We expect an exception to be raised if the same volume
        # can be accessed through multiple paths.
        device_paths = [mock.sentinel.dev_path_0,
                        mock.sentinel.dev_path_1]
        self.assertRaises(exception.BrickException,
                          self._connector._check_device_paths,
                          device_paths)

    @mock.patch.object(fake_win_conn.FakeWindowsConnector,
                       'get_volume_paths')
    def test_extend_volume(self, mock_get_vol_paths):
        mock_vol_paths = [mock.sentinel.dev_path]
        mock_get_vol_paths.return_value = mock_vol_paths

        self._connector.extend_volume(mock.sentinel.conn_props)

        mock_get_vol_paths.assert_called_once_with(mock.sentinel.conn_props)
        mock_get_dev_num = self._diskutils.get_device_number_from_device_name
        mock_get_dev_num.assert_called_once_with(mock.sentinel.dev_path)
        self._diskutils.refresh_disk.assert_called_once_with(
            mock_get_dev_num.return_value)

    @mock.patch.object(fake_win_conn.FakeWindowsConnector,
                       'get_volume_paths')
    def test_extend_volume_missing_path(self, mock_get_vol_paths):
        mock_get_vol_paths.return_value = []

        self.assertRaises(exception.NotFound,
                          self._connector.extend_volume,
                          mock.sentinel.conn_props)

        mock_get_vol_paths.assert_called_once_with(mock.sentinel.conn_props)



# (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from os_brick.initiator.connectors import drbd
from os_brick.tests.initiator import test_connector


class DRBDConnectorTestCase(test_connector.ConnectorTestCase):

    RESOURCE_TEMPLATE = '''
        resource r0 {
            on host1 {
            }
            net {
                shared-secret "%(shared-secret)s";
            }
        }
'''

    def setUp(self):
        super(DRBDConnectorTestCase, self).setUp()

        self.connector = drbd.DRBDConnector(
            None, execute=self._fake_exec)

        self.execs = []

    def _fake_exec(self, *cmd, **kwargs):
        self.execs.append(cmd)

        # out, err
        return ('', '')

    def test_get_connector_properties(self):
        props = drbd.DRBDConnector.get_connector_properties(
            'sudo', multipath=True, enforce_multipath=True)

        expected_props = {}
        self.assertEqual(expected_props, props)

    def test_connect_volume(self):
        """Test connect_volume."""

        cprop = {
            'provider_auth': 'my-secret',
            'config': self.RESOURCE_TEMPLATE,
            'name': 'my-precious',
            'device': '/dev/drbd951722',
            'data': {},
        }

        res = self.connector.connect_volume(cprop)

        self.assertEqual(cprop['device'], res['path'])
        self.assertEqual('adjust', self.execs[0][1])
        self.assertEqual(cprop['name'], self.execs[0][4])

    def test_disconnect_volume(self):
        """Test the disconnect volume case."""

        cprop = {
            'provider_auth': 'my-secret',
            'config': self.RESOURCE_TEMPLATE,
            'name': 'my-precious',
            'device': '/dev/drbd951722',
            'data': {},
        }
        dev_info = {}

        self.connector.disconnect_volume(cprop, dev_info)

        self.assertEqual('down', self.execs[0][1])

    def test_extend_volume(self):
        cprop = {'name': 'something'}
        self.assertRaises(NotImplementedError,
                          self.connector.extend_volume,
                          cprop)



# Copyright 2016 Cloudbase Solutions Srl
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import collections
import time

from os_win import utilsfactory
from oslo_log import log as logging

from os_brick import exception
from os_brick.initiator.windows import base as win_conn_base
from os_brick import utils

LOG = logging.getLogger(__name__)


class WindowsFCConnector(win_conn_base.BaseWindowsConnector):
    def __init__(self, *args, **kwargs):
        super(WindowsFCConnector, self).__init__(*args, **kwargs)
        self._fc_utils = utilsfactory.get_fc_utils()

    @staticmethod
    def get_connector_properties(*args, **kwargs):
        props = {}

        fc_utils = utilsfactory.get_fc_utils()
        fc_utils.refresh_hba_configuration()
        fc_hba_ports = fc_utils.get_fc_hba_ports()

        if fc_hba_ports:
            wwnns = []
            wwpns = []
            for port in fc_hba_ports:
                wwnns.append(port['node_name'])
                wwpns.append(port['port_name'])
            props['wwpns'] = wwpns
            props['wwnns'] = list(set(wwnns))
        return props

    @utils.trace
    def connect_volume(self, connection_properties):
        volume_paths = self.get_volume_paths(connection_properties)
        if not volume_paths:
            raise exception.NoFibreChannelVolumeDeviceFound()

        device_path = volume_paths[0]
        device_number = self._diskutils.get_device_number_from_device_name(
            device_path)
        scsi_wwn = self._get_scsi_wwn(device_number)
        device_info = {'type': 'block',
                       'path': device_path,
                       'number': device_number,
                       'scsi_wwn': scsi_wwn}
        return device_info

    @utils.trace
    def get_volume_paths(self, connection_properties):
        # Returns a list containing at most one disk path such as
        # \\.\PhysicalDrive4.
        #
        # If multipath is used and the MPIO service is properly configured
        # to claim the disks, we'll still get a single device path, having
        # the same format, which will be used for all the IO operations.
        disk_paths = set()

        for attempt in range(self.device_scan_attempts):
            self._diskutils.rescan_disks()
            volume_mappings = self._get_fc_volume_mappings(
                connection_properties)
            LOG.debug("Retrieved volume mappings %(vol_mappings)s "
                      "for volume %(conn_props)s",
                      dict(vol_mappings=volume_mappings,
                           conn_props=connection_properties))

            # Because of MPIO, we may not be able to get the device name
            # from a specific mapping if the disk was accessed through
            # an other HBA at that moment. In that case, the device name
            # will show up as an empty string.
            for mapping in volume_mappings:
                device_name = mapping['device_name']
                if device_name:
                    disk_paths.add(device_name)

            if disk_paths:
                break

            time.sleep(self.device_scan_interval)

        self._check_device_paths(disk_paths)
        return list(disk_paths)

    def _get_fc_volume_mappings(self, connection_properties):
        # Note(lpetrut): All the WWNs returned by os-win are upper case.
        target_wwpns = [wwpn.upper()
                        for wwpn in connection_properties['target_wwn']]
        target_lun = connection_properties['target_lun']

        volume_mappings = []
        hba_mappings = self._get_fc_hba_mappings()
        for node_name in hba_mappings:
            target_mappings = self._fc_utils.get_fc_target_mappings(node_name)
            for mapping in target_mappings:
                if (mapping['port_name'] in target_wwpns
                        and mapping['lun'] == target_lun):
                    volume_mappings.append(mapping)

        return volume_mappings

    def _get_fc_hba_mappings(self):
        mappings = collections.defaultdict(list)
        fc_hba_ports = self._fc_utils.get_fc_hba_ports()
        for port in fc_hba_ports:
            mappings[port['node_name']].append(port['port_name'])
        return mappings

    @utils.trace
    def disconnect_volume(self, connection_properties,
                          force=False, ignore_errors=False):
        pass



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import tempfile

from oslo_concurrency import processutils as putils

from os_brick.initiator.connectors import base
from os_brick import utils


class DRBDConnector(base.BaseLinuxConnector):
    """"Connector class to attach/detach DRBD resources."""

    def __init__(self, root_helper, driver=None,
                 execute=putils.execute, *args, **kwargs):

        super(DRBDConnector, self).__init__(root_helper, driver=driver,
                                            execute=execute, *args, **kwargs)

        self._execute = execute
        self._root_helper = root_helper

    @staticmethod
    def get_connector_properties(root_helper, *args, **kwargs):
        """The DRBD connector properties."""
        return {}

    def check_valid_device(self, path, run_as_root=True):
        """Verify an existing volume."""
        # TODO(linbit): check via drbdsetup first, to avoid blocking/hanging
        # in case of network problems?

        return super(DRBDConnector, self).check_valid_device(path, run_as_root)

    def get_all_available_volumes(self, connection_properties=None):

        base = "/dev/"
        blkdev_list = []

        for e in os.listdir(base):
            path = base + e
            if os.path.isblk(path):
                blkdev_list.append(path)

        return blkdev_list

    def _drbdadm_command(self, cmd, data_dict, sh_secret):
        # TODO(linbit): Write that resource file to a permanent location?
        tmp = tempfile.NamedTemporaryFile(suffix="res", delete=False, mode="w")
        try:
            kv = {'shared-secret': sh_secret}
            tmp.write(data_dict['config'] % kv)
            tmp.close()

            (out, err) = self._execute('drbdadm', cmd,
                                       "-c", tmp.name,
                                       data_dict['name'],
                                       run_as_root=True,
                                       root_helper=self._root_helper)
        finally:
            os.unlink(tmp.name)

        return (out, err)

    @utils.trace
    def connect_volume(self, connection_properties):
        """Attach the volume."""

        self._drbdadm_command("adjust", connection_properties,
                              connection_properties['provider_auth'])

        device_info = {
            'type': 'block',
            'path': connection_properties['device'],
        }

        return device_info

    @utils.trace
    def disconnect_volume(self, connection_properties, device_info,
                          force=False, ignore_errors=False):
        """Detach the volume."""

        self._drbdadm_command("down", connection_properties,
                              connection_properties['provider_auth'])

    def get_volume_paths(self, connection_properties):
        path = connection_properties['device']
        return [path]

    def get_search_path(self):
        # TODO(linbit): is it allowed to return "/dev", or is that too broad?
        return None

    def extend_volume(self, connection_properties):
        # TODO(walter-boring): is this possible?
        raise NotImplementedError



# (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from os_brick import exception
from os_brick.initiator.connectors import sheepdog
from os_brick.initiator import linuxsheepdog
from os_brick.tests.initiator import test_connector


class SheepdogConnectorTestCase(test_connector.ConnectorTestCase):

    def setUp(self):
        super(SheepdogConnectorTestCase, self).setUp()

        self.hosts = ['fake_hosts']
        self.ports = ['fake_ports']
        self.volume = 'fake_volume'

        self.connection_properties = {
            'hosts': self.hosts,
            'name': self.volume,
            'ports': self.ports,
        }

    def test_get_connector_properties(self):
        props = sheepdog.SheepdogConnector.get_connector_properties(
            'sudo', multipath=True, enforce_multipath=True)

        expected_props = {}
        self.assertEqual(expected_props, props)

    def test_get_search_path(self):
        sd_connector = sheepdog.SheepdogConnector(None)
        path = sd_connector.get_search_path()
        self.assertIsNone(path)

    def test_get_volume_paths(self):
        sd_connector = sheepdog.SheepdogConnector(None)
        expected = []
        actual = sd_connector.get_volume_paths(self.connection_properties)
        self.assertEqual(expected, actual)

    def test_connect_volume(self):
        """Test the connect volume case."""
        sd_connector = sheepdog.SheepdogConnector(None)
        device_info = sd_connector.connect_volume(self.connection_properties)

        # Ensure expected object is returned correctly
        self.assertIsInstance(device_info['path'],
                              linuxsheepdog.SheepdogVolumeIOWrapper)

    @mock.patch.object(linuxsheepdog.SheepdogVolumeIOWrapper, 'close')
    def test_disconnect_volume(self, volume_close):
        """Test the disconnect volume case."""
        sd_connector = sheepdog.SheepdogConnector(None)
        device_info = sd_connector.connect_volume(self.connection_properties)
        sd_connector.disconnect_volume(self.connection_properties, device_info)

        self.assertEqual(1, volume_close.call_count)

    def test_disconnect_volume_with_invalid_handle(self):
        """Test the disconnect volume case with invalid handle."""
        sd_connector = sheepdog.SheepdogConnector(None)
        device_info = {'path': 'fake_handle'}
        self.assertRaises(exception.InvalidIOHandleObject,
                          sd_connector.disconnect_volume,
                          self.connection_properties,
                          device_info)

    def test_extend_volume(self):
        sd_connector = sheepdog.SheepdogConnector(None)
        self.assertRaises(NotImplementedError,
                          sd_connector.extend_volume,
                          self.connection_properties)



# Copyright (c) 2015 Scality
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import errno

import mock

from os_brick.initiator import host_driver
from os_brick.tests import base


class HostDriverTestCase(base.TestCase):

    def test_get_all_block_devices(self):
        fake_dev = ['device1', 'device2']
        expected = ['/dev/disk/by-path/' + dev for dev in fake_dev]

        driver = host_driver.HostDriver()
        with mock.patch('os.listdir', return_value=fake_dev):
            actual = driver.get_all_block_devices()

        self.assertEqual(expected, actual)

    def test_get_all_block_devices_when_oserror_is_enoent(self):
        driver = host_driver.HostDriver()
        oserror = OSError(errno.ENOENT, "")
        with mock.patch('os.listdir', side_effect=oserror):
            block_devices = driver.get_all_block_devices()

        self.assertEqual([], block_devices)

    def test_get_all_block_devices_when_oserror_is_not_enoent(self):
        driver = host_driver.HostDriver()
        oserror = OSError(errno.ENOMEM, "")
        with mock.patch('os.listdir', side_effect=oserror):
            self.assertRaises(OSError, driver.get_all_block_devices)



# (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import mock

from os_brick.initiator.connectors import remotefs
from os_brick.remotefs import remotefs as remotefs_client
from os_brick.tests.initiator import test_connector


class RemoteFsConnectorTestCase(test_connector.ConnectorTestCase):
    """Test cases for Remote FS initiator class."""
    TEST_DEV = '172.18.194.100:/var/nfs'
    TEST_PATH = '/mnt/test/df0808229363aad55c27da50c38d6328'
    TEST_BASE = '/mnt/test'
    TEST_NAME = '9c592d52-ce47-4263-8c21-4ecf3c029cdb'

    def setUp(self):
        super(RemoteFsConnectorTestCase, self).setUp()
        self.connection_properties = {
            'export': self.TEST_DEV,
            'name': self.TEST_NAME}
        self.connector = remotefs.RemoteFsConnector(
            'nfs', root_helper='sudo',
            nfs_mount_point_base=self.TEST_BASE,
            nfs_mount_options='vers=3')

    @mock.patch('os_brick.remotefs.remotefs.ScalityRemoteFsClient')
    def test_init_with_scality(self, mock_scality_remotefs_client):
        remotefs.RemoteFsConnector('scality', root_helper='sudo')
        self.assertEqual(1, mock_scality_remotefs_client.call_count)

    def test_get_connector_properties(self):
        props = remotefs.RemoteFsConnector.get_connector_properties(
            'sudo', multipath=True, enforce_multipath=True)

        expected_props = {}
        self.assertEqual(expected_props, props)

    def test_get_search_path(self):
        expected = self.TEST_BASE
        actual = self.connector.get_search_path()
        self.assertEqual(expected, actual)

    @mock.patch.object(remotefs_client.RemoteFsClient, 'mount')
    def test_get_volume_paths(self, mock_mount):
        path = ("%(path)s/%(name)s" % {'path': self.TEST_PATH,
                                       'name': self.TEST_NAME})
        expected = [path]
        actual = self.connector.get_volume_paths(self.connection_properties)
        self.assertEqual(expected, actual)

    @mock.patch.object(remotefs_client.RemoteFsClient, 'mount')
    @mock.patch.object(remotefs_client.RemoteFsClient, 'get_mount_point',
                       return_value="something")
    def test_connect_volume(self, mount_point_mock, mount_mock):
        """Test the basic connect volume case."""
        self.connector.connect_volume(self.connection_properties)

    def test_disconnect_volume(self):
        """Nothing should happen here -- make sure it doesn't blow up."""
        self.connector.disconnect_volume(self.connection_properties, {})

    def test_extend_volume(self):
        self.assertRaises(NotImplementedError,
                          self.connector.extend_volume,
                          self.connection_properties)



# (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import mock
import os
import tempfile

from os_brick import exception
from os_brick.initiator.connectors import huawei
from os_brick.tests.initiator import test_connector


class HuaweiStorHyperConnectorTestCase(test_connector.ConnectorTestCase):
    """Test cases for StorHyper initiator class."""

    attached = False

    def setUp(self):
        super(HuaweiStorHyperConnectorTestCase, self).setUp()
        self.fake_sdscli_file = tempfile.mktemp()
        self.addCleanup(os.remove, self.fake_sdscli_file)
        newefile = open(self.fake_sdscli_file, 'w')
        newefile.write('test')
        newefile.close()

        self.connector = huawei.HuaweiStorHyperConnector(
            None, execute=self.fake_execute)
        self.connector.cli_path = self.fake_sdscli_file
        self.connector.iscliexist = True

        self.connector_fail = huawei.HuaweiStorHyperConnector(
            None, execute=self.fake_execute_fail)
        self.connector_fail.cli_path = self.fake_sdscli_file
        self.connector_fail.iscliexist = True

        self.connector_nocli = huawei.HuaweiStorHyperConnector(
            None, execute=self.fake_execute_fail)
        self.connector_nocli.cli_path = self.fake_sdscli_file
        self.connector_nocli.iscliexist = False

        self.connection_properties = {
            'access_mode': 'rw',
            'qos_specs': None,
            'volume_id': 'volume-b2911673-863c-4380-a5f2-e1729eecfe3f'
        }

        self.device_info = {'type': 'block',
                            'path': '/dev/vdxxx'}
        HuaweiStorHyperConnectorTestCase.attached = False

    def fake_execute(self, *cmd, **kwargs):
        method = cmd[2]
        self.cmds.append(" ".join(cmd))
        if 'attach' == method:
            HuaweiStorHyperConnectorTestCase.attached = True
            return 'ret_code=0', None
        if 'querydev' == method:
            if HuaweiStorHyperConnectorTestCase.attached:
                return 'ret_code=0\ndev_addr=/dev/vdxxx', None
            else:
                return 'ret_code=1\ndev_addr=/dev/vdxxx', None
        if 'detach' == method:
            HuaweiStorHyperConnectorTestCase.attached = False
            return 'ret_code=0', None

    def fake_execute_fail(self, *cmd, **kwargs):
        method = cmd[2]
        self.cmds.append(" ".join(cmd))
        if 'attach' == method:
            HuaweiStorHyperConnectorTestCase.attached = False
            return 'ret_code=330151401', None
        if 'querydev' == method:
            if HuaweiStorHyperConnectorTestCase.attached:
                return 'ret_code=0\ndev_addr=/dev/vdxxx', None
            else:
                return 'ret_code=1\ndev_addr=/dev/vdxxx', None
        if 'detach' == method:
            HuaweiStorHyperConnectorTestCase.attached = True
            return 'ret_code=330155007', None

    def test_get_connector_properties(self):
        props = huawei.HuaweiStorHyperConnector.get_connector_properties(
            'sudo', multipath=True, enforce_multipath=True)

        expected_props = {}
        self.assertEqual(expected_props, props)

    def test_get_search_path(self):
        actual = self.connector.get_search_path()
        self.assertIsNone(actual)

    @mock.patch.object(huawei.HuaweiStorHyperConnector,
                       '_query_attached_volume')
    def test_get_volume_paths(self, mock_query_attached):
        path = self.device_info['path']
        mock_query_attached.return_value = {'ret_code': 0,
                                            'dev_addr': path}

        expected = [path]
        actual = self.connector.get_volume_paths(self.connection_properties)
        self.assertEqual(expected, actual)

    def test_connect_volume(self):
        """Test the basic connect volume case."""

        retval = self.connector.connect_volume(self.connection_properties)
        self.assertEqual(self.device_info, retval)

        expected_commands = [self.fake_sdscli_file + ' -c attach'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f',
                             self.fake_sdscli_file + ' -c querydev'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f']

        self.assertEqual(expected_commands, self.cmds)

    def test_disconnect_volume(self):
        """Test the basic disconnect volume case."""
        self.connector.connect_volume(self.connection_properties)
        self.assertEqual(True, HuaweiStorHyperConnectorTestCase.attached)
        self.connector.disconnect_volume(self.connection_properties,
                                         self.device_info)
        self.assertEqual(False, HuaweiStorHyperConnectorTestCase.attached)

        expected_commands = [self.fake_sdscli_file + ' -c attach'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f',
                             self.fake_sdscli_file + ' -c querydev'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f',
                             self.fake_sdscli_file + ' -c detach'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f']

        self.assertEqual(expected_commands, self.cmds)

    def test_is_volume_connected(self):
        """Test if volume connected to host case."""
        self.connector.connect_volume(self.connection_properties)
        self.assertEqual(True, HuaweiStorHyperConnectorTestCase.attached)
        is_connected = self.connector.is_volume_connected(
            'volume-b2911673-863c-4380-a5f2-e1729eecfe3f')
        self.assertEqual(HuaweiStorHyperConnectorTestCase.attached,
                         is_connected)
        self.connector.disconnect_volume(self.connection_properties,
                                         self.device_info)
        self.assertEqual(False, HuaweiStorHyperConnectorTestCase.attached)
        is_connected = self.connector.is_volume_connected(
            'volume-b2911673-863c-4380-a5f2-e1729eecfe3f')
        self.assertEqual(HuaweiStorHyperConnectorTestCase.attached,
                         is_connected)

        expected_commands = [self.fake_sdscli_file + ' -c attach'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f',
                             self.fake_sdscli_file + ' -c querydev'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f',
                             self.fake_sdscli_file + ' -c querydev'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f',
                             self.fake_sdscli_file + ' -c detach'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f',
                             self.fake_sdscli_file + ' -c querydev'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f']

        self.assertEqual(expected_commands, self.cmds)

    def test__analyze_output(self):
        cliout = 'ret_code=0\ndev_addr=/dev/vdxxx\nret_desc="success"'
        analyze_result = {'dev_addr': '/dev/vdxxx',
                          'ret_desc': '"success"',
                          'ret_code': '0'}
        result = self.connector._analyze_output(cliout)
        self.assertEqual(analyze_result, result)

    def test_connect_volume_fail(self):
        """Test the fail connect volume case."""
        self.assertRaises(exception.BrickException,
                          self.connector_fail.connect_volume,
                          self.connection_properties)
        expected_commands = [self.fake_sdscli_file + ' -c attach'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f']
        self.assertEqual(expected_commands, self.cmds)

    def test_disconnect_volume_fail(self):
        """Test the fail disconnect volume case."""
        self.connector.connect_volume(self.connection_properties)
        self.assertEqual(True, HuaweiStorHyperConnectorTestCase.attached)
        self.assertRaises(exception.BrickException,
                          self.connector_fail.disconnect_volume,
                          self.connection_properties,
                          self.device_info)

        expected_commands = [self.fake_sdscli_file + ' -c attach'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f',
                             self.fake_sdscli_file + ' -c querydev'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f',
                             self.fake_sdscli_file + ' -c detach'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f']

        self.assertEqual(expected_commands, self.cmds)

    def test_connect_volume_nocli(self):
        """Test the fail connect volume case."""
        self.assertRaises(exception.BrickException,
                          self.connector_nocli.connect_volume,
                          self.connection_properties)

    def test_disconnect_volume_nocli(self):
        """Test the fail disconnect volume case."""
        self.connector.connect_volume(self.connection_properties)
        self.assertEqual(True, HuaweiStorHyperConnectorTestCase.attached)
        self.assertRaises(exception.BrickException,
                          self.connector_nocli.disconnect_volume,
                          self.connection_properties,
                          self.device_info)
        expected_commands = [self.fake_sdscli_file + ' -c attach'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f',
                             self.fake_sdscli_file + ' -c querydev'
                             ' -v volume-b2911673-863c-4380-a5f2-e1729eecfe3f']
        self.assertEqual(expected_commands, self.cmds)

    def test_extend_volume(self):
        self.assertRaises(NotImplementedError,
                          self.connector.extend_volume,
                          self.connection_properties)



# Copyright (c) 2017 Veritas Technologies LLC
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from oslo_concurrency import processutils

from os_brick import exception
from os_brick.initiator.connectors import vrtshyperscale
from os_brick.tests.initiator import test_connector

DEVICE_NAME = '{8ee71c33-dcd0-4267-8f2b-e0742ecabe9f}'
DEVICE_PATH = '/dev/8ee71c33-dcd0-4267-8f2b-e0742ec'


class HyperScaleConnectorTestCase(test_connector.ConnectorTestCase):
    """Test cases for Veritas HyperScale os-brick connector."""

    def _fake_execute_success(self, *cmd, **kwargs):
        """Mock successful execution of hscli"""
        result_json = ""
        err = 0
        args = json.loads(cmd[1])
        if args['operation'] == 'connect_volume':
            result = {}
            payload = {}
            payload['vsa_ip'] = '192.0.2.2'
            payload['refl_factor'] = '2'
            payload['refl_targets'] = '192.0.2.3,192.0.2.4'
            result['payload'] = payload
            result_json = json.dumps(result)
        return (result_json, err)

    def _fake_execute_hscli_missing(self, *cmd, **kwargs):
        """Mock attempt to execute missing hscli"""
        raise processutils.ProcessExecutionError()
        return ("", 0)

    def _fake_execute_hscli_err(self, *cmd, **kwargs):
        """Mock hscli returning error"""
        result_json = ""
        err = 'fake_hscli_error_msg'
        return (result_json, err)

    def _fake_execute_hscli_res_inval(self, *cmd, **kwargs):
        """Mock hscli returning unexpected values"""
        result_json = ""
        err = 0
        result = {}
        payload = {}
        payload['unexpected'] = 'junk'
        result['payload'] = payload
        result_json = json.dumps(result)
        return (result_json, err)

    def test_connect_volume_normal(self):
        """Test results of successful connect_volume()"""
        connector = vrtshyperscale.HyperScaleConnector(
            'sudo', execute=self._fake_execute_success)
        fake_connection_properties = {
            'name': DEVICE_NAME
        }
        device_info = connector.connect_volume(fake_connection_properties)

        self.assertEqual('192.0.2.2', device_info['vsa_ip'])
        self.assertEqual('2', device_info['refl_factor'])
        self.assertEqual('192.0.2.3,192.0.2.4', device_info['refl_targets'])
        self.assertEqual(DEVICE_PATH, device_info['path'])

    def test_connect_volume_arg_missing(self):
        """Test connect_volume with missing missing arguments"""
        connector = vrtshyperscale.HyperScaleConnector(
            'sudo', execute=self._fake_execute_success)
        fake_connection_properties = {}
        self.assertRaises(exception.BrickException,
                          connector.connect_volume,
                          fake_connection_properties)

    def test_connect_volume_hscli_missing(self):
        """Test connect_volume that can't call hscli"""
        connector = vrtshyperscale.HyperScaleConnector(
            'sudo', execute=self._fake_execute_hscli_missing)
        fake_connection_properties = {
            'name': DEVICE_NAME
        }
        self.assertRaises(exception.BrickException,
                          connector.connect_volume,
                          fake_connection_properties)

    def test_connect_volume_hscli_err(self):
        """Test connect_volume when hscli returns an error"""
        connector = vrtshyperscale.HyperScaleConnector(
            'sudo', execute=self._fake_execute_hscli_err)
        fake_connection_properties = {
            'name': DEVICE_NAME
        }
        self.assertRaises(exception.BrickException,
                          connector.connect_volume,
                          fake_connection_properties)

    def test_connect_volume_hscli_res_inval(self):
        """Test connect_volume if hscli returns an invalid result"""
        connector = vrtshyperscale.HyperScaleConnector(
            'sudo', execute=self._fake_execute_hscli_res_inval)
        fake_connection_properties = {
            'name': DEVICE_NAME
        }
        self.assertRaises(exception.BrickException,
                          connector.connect_volume,
                          fake_connection_properties)

    def test_disconnect_volume_normal(self):
        """Test successful disconnect_volume call"""
        connector = vrtshyperscale.HyperScaleConnector(
            'sudo', execute=self._fake_execute_success)
        fake_connection_properties = {
            'name': DEVICE_NAME
        }
        fake_device_info = {}
        connector.disconnect_volume(fake_connection_properties,
                                    fake_device_info)

    def test_disconnect_volume_arg_missing(self):
        """Test disconnect_volume with missing arguments"""
        connector = vrtshyperscale.HyperScaleConnector(
            'sudo', execute=self._fake_execute_success)
        fake_connection_properties = {}
        fake_device_info = {}
        self.assertRaises(exception.BrickException,
                          connector.disconnect_volume,
                          fake_connection_properties,
                          fake_device_info)
